package cn.superdesk.canal.client.client;

import cn.superdesk.canal.client.handler.MessageHandler;
import com.alibaba.otter.canal.client.kafka.KafkaCanalConnector;
import com.alibaba.otter.canal.protocol.Message;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.TimeUnit;

/**
 * @author xc
 * @since 2023/12/21 16:00
 */
@SuppressWarnings("unchecked,rawtypes")
public class AbstractCanalClient implements CanalClient {
    protected final Logger log = LoggerFactory.getLogger(KafkaCanalClient.class);

    private KafkaCanalConnector connector;
    private MessageHandler messageHandler;
    protected static volatile boolean running = false;

    private Thread thread = null;

    protected Thread.UncaughtExceptionHandler handler = (t, e) -> log.error("parse events has an error", e);

    protected Integer batchSize = 1;
    protected Long timeout = 1L;
    protected TimeUnit unit = TimeUnit.SECONDS;
    protected String filter = StringUtils.EMPTY;


    @Override
    public void start() {
        log.info("start canal client");
        thread = new Thread(this::process);
        thread.setName("canal-client-thread");
        thread.setUncaughtExceptionHandler(handler);
        running = true;
        thread.start();
    }

    @Override
    public void stop() {
        if (!running) {
            return;
        }
        running = false;
        if (thread != null) {
            try {
                thread.join();
            } catch (InterruptedException e) {
                log.error("canal client stop error", e);
            }
        }
    }

    @Override
    public void process() {
        while (running) {
            try {
                connector.connect();
                connector.subscribe(filter);
                while (running) {
                    Message message = connector.getWithoutAck(batchSize, timeout, unit);
                    long batchId = message.getId();

                    if (batchId != -1 && !message.getEntries().isEmpty()) {
                        messageHandler.handleMessage(message);
                    }
                    connector.ack(batchId);
                }
            } catch (Exception e) {
                log.error("canal client异常", e);
            }
        }
        connector.unsubscribe();
        connector.disconnect();
    }

    public KafkaCanalConnector getConnector() {
        return connector;
    }

    public void setConnector(KafkaCanalConnector connector) {
        this.connector = connector;
    }

    public MessageHandler<?> getMessageHandler() {
        return messageHandler;
    }

    public void setMessageHandler(MessageHandler<?> messageHandler) {
        this.messageHandler = messageHandler;
    }
}
