package cn.superdesk.canal.client.handler;

import cn.superdesk.canal.client.util.HandlerUtil;
import com.alibaba.otter.canal.protocol.CanalEntry;
import com.alibaba.otter.canal.protocol.FlatMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public abstract class AbstractFlatMessageHandler implements MessageHandler<FlatMessage> {


	private final Map<String, EntryHandler<?>> tableHandlerMap;


	private final RowDataHandler<List<Map<String, String>>> rowDataHandler;


	private final Logger logger = LoggerFactory.getLogger(AbstractFlatMessageHandler.class);


	public AbstractFlatMessageHandler(List<? extends EntryHandler<?>> entryHandlers, RowDataHandler<List<Map<String, String>>> rowDataHandler) {
		this.tableHandlerMap = HandlerUtil.getTableHandlerMap(entryHandlers);
		this.rowDataHandler = rowDataHandler;
	}

	@Override
	public void handleMessage(FlatMessage flatMessage) {
		List<Map<String, String>> data = flatMessage.getData();
		if (data != null && !data.isEmpty()) {
			for (int i = 0; i < data.size(); i++) {
				CanalEntry.EventType      eventType = CanalEntry.EventType.valueOf(flatMessage.getType());
				List<Map<String, String>> maps;
				if (eventType.equals(CanalEntry.EventType.UPDATE)) {
					Map<String, String> map    = data.get(i);
					Map<String, String> oldMap = flatMessage.getOld().get(i);
					maps = Stream.of(map, oldMap).collect(Collectors.toList());
				} else {
					maps = Stream.of(data.get(i)).collect(Collectors.toList());
				}
				try {
					EntryHandler<?> entryHandler = HandlerUtil.getEntryHandler(tableHandlerMap, flatMessage.getTable());
					if (entryHandler != null) {
						rowDataHandler.handlerRowData(maps, entryHandler, eventType);
					}
				} catch (Exception e) {
					throw new RuntimeException("parse event has an error , data:" + data.toString(), e);
				}
			}
		}
	}
}
