package cn.superdesk.canal2kafka.spring.boot.autoconfigure;

import cn.superdesk.canal.client.client.KafkaCanalClient;
import cn.superdesk.canal.client.factory.MapColumnModelFactory;
import cn.superdesk.canal.client.handler.*;
import com.alibaba.otter.canal.protocol.FlatMessage;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;

import java.util.List;
import java.util.Map;

/**
 * canal kafka客户端自动配置类
 * @author xc
 * @since 2023/12/21 15:19
 */
@EnableConfigurationProperties(CanalProperties.class)
@Configuration
@ConditionalOnBean(value = {EntryHandler.class})
@ConditionalOnProperty(value = CanalProperties.CANAL_MODE, havingValue = "kafka")
@Import(ThreadPoolAutoConfiguration.class)
public class CanalKafkaClientAutoConfiguration {
    private final CanalProperties canalProperties;


    public CanalKafkaClientAutoConfiguration(CanalProperties canalProperties) {
        this.canalProperties = canalProperties;
    }

    @Bean
    public RowDataHandler<List<Map<String, String>>> rowDataHandler() {
        return new MapRowDataHandlerImpl(new MapColumnModelFactory());
    }

    @Bean
    public MessageHandler<FlatMessage> messageHandler(RowDataHandler<List<Map<String, String>>> rowDataHandler, List<EntryHandler<?>> entryHandlers) {
        return new SyncFlatMessageHandlerImpl(entryHandlers, rowDataHandler);
    }



    @Bean(initMethod = "start", destroyMethod = "stop")
    public KafkaCanalClient canalKafkaClient(MessageHandler<FlatMessage> messageHandler) {
        return KafkaCanalClient.builder()
                .servers(canalProperties.getServers())
                .groupId(canalProperties.getGroupId())
                .topic(canalProperties.getDestination())
                .messageHandler(messageHandler)
                .filter(canalProperties.getFilter())
                .batchSize(canalProperties.getBatchSize())
                .timeout(canalProperties.getTimeout())
                .unit(canalProperties.getUnit())
                .build();
    }

}
